<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2005 Anatoliy Balabanov (balabanov@i-tribe.com),
*      2008 Alexey Gafiulov <gafiulov@i-tribe.com>
*      2010 Andrey Krotov <krotov@i-tribe.com>
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/
/**
 * Plugin 'TypoScript code' for the 'typoscript_code' extension.
 *
 * @author	Anatoliy Balabanov <balabanov@i-tribe.com>
 * @author	Alexey Gafiulov <gafiulov@i-tribe.com>
 * @author	Andrey Krotov <krotov@i-tribe.com>
 */


require_once(PATH_tslib.'class.tslib_pibase.php');

class tx_typoscriptcode_pi1 extends tslib_pibase {

	public $prefixId = 'tx_typoscriptcode_pi1'; // Same as class name

	public $scriptRelPath = 'pi1/class.tx_typoscriptcode_pi1.php'; // Path to this script relative to the extension dir.

	public $extKey = 'typoscript_code'; // The extension key.

	public $pi_checkCHash = TRUE;

	/**
	 * array with LL values
	 *
	 * @var array
	 */
	public $lConf = NULL;

	/**
	 * main parse class
	 *
	 * @var t3lib_TSparser
	 */
	protected $_parser;

	/**
	 * support parse class
	 *
	 * @var t3lib_TSparser
	 */
	protected $_constParser;

	/**
	 * Matching TypoScript conditions
	 *
	 * @var t3lib_matchCondition
	 */
	protected $_matchCondition;

	/**
	* Typoscript code for insert and parsing typoscript from tt_content and returning content
	*
	* @return 	string 	content
	*/
	public function main($content,$conf) {
		$this->conf=$conf;
		$this->pi_setPiVarDefaults();
		$this->pi_loadLL();
		$this->init();

		$setup = $this->scriptParser($this->lConf['code_text']);		
		$this->_tryChangeExtType();
		
		return $this->cObj->cObjGet($setup, 'typoscript_code_proc.');		
	}

	/**
	* Load typoscript from flexValue , parse, and return conf - array
	* @param 	string 	string typoscript
	*
	* @return 	array 	typoscript configuration array
	*/
	public function scriptParser($script="") {

		// get constants
		$this->_constParser->parse(implode("\n", $GLOBALS['TSFE']->tmpl->constants), $this->_matchCondition);

		// recursive substitution of constants (up to 10 nested levels)
		for ($i = 0; $i < 10; $i++) {
			$old_script = $script;
			$script = preg_replace_callback('/\{\$(.[^}]*)\}/', array($this, 'substituteConstantsCallBack'), $script);
			if ($old_script == $script) {
				break;
			}
		}

		if (is_array($GLOBALS['TSFE']->tmpl->setup)) {
			foreach ($GLOBALS['TSFE']->tmpl->setup as $tsObjectKey => $tsObjectValue) {
				if ($tsObjectKey !== intval($tsObjectKey)) {
					$this->_parser->setup[$tsObjectKey] = $tsObjectValue;
				}
			}
		}

		$this->_parser->parse($script, $this->_matchCondition);

		return $this->_parser->setup;
	}

	/**
	 * change ext type to user_int if necessary
	 *
	 */
	protected function _tryChangeExtType() {
		if(isset($this->_parser->sections) && is_array($this->_parser->sections) && count($this->_parser->sections) > 0) {
			$this->cObj->convertToUserIntObject();
		}
	}

	/**
	* Call back method for preg_replace_callback in substituteConstants
	*
	* @param	array		Regular expression matches
	* @return	string		Replacement
	* @see substituteConstants()
	*/
	public function substituteConstantsCallBack($matches) {
		$s = $this->_constParser->getVal($matches[1], $this->_constParser->setup);
		return isset($s[0]) ? $s[0] : $matches[0];
	}

	/**
	* Init  flexForm and Flex value.
	*
	* @return 	void.
	*/
	public function init() {
		$this->pi_initPIflexForm();
		$this->lConf = array();
		$piFlexForm = $this->cObj->data['pi_flexform'];
		foreach($piFlexForm['data'] as $sheet => $data) {
			foreach($data as $lang => $value) {
				foreach($value as $key => $val) {
					$this->lConf[$key] = $this->pi_getFFvalue($piFlexForm, $key, $sheet);
				}
			}
		}

		$this->_parser = t3lib_div::makeInstance('t3lib_TSparser');
		$this->_constParser = t3lib_div::makeInstance('t3lib_TSparser');
		$this->_matchCondition = t3lib_div::makeInstance('t3lib_matchCondition');
	}

}

if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/typoscript_code/pi1/class.tx_typoscriptcode_pi1.php'])	{
	include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/typoscript_code/pi1/class.tx_typoscriptcode_pi1.php']);
}

?>